@@ -14,8 +14,6 @@
 # include <sys/mman.h>
 #endif
 
-#include "utils.h"
-#include "randombytes.h"
 #ifdef _WIN32
 # include <windows.h>
 # include <wincrypt.h>
@@ -23,16 +21,42 @@
 # include <unistd.h>
 #endif
 
+#ifndef HAVE_C_VARARRAYS
+# ifdef HAVE_ALLOCA_H
+#  include <alloca.h>
+# elif !defined(alloca)
+#  if defined(__clang__) || defined(__GNUC__)
+#   define alloca __builtin_alloca
+#  elif defined _AIX
+#   define alloca __alloca
+#  elif defined _MSC_VER
+#   include <malloc.h>
+#   define alloca _alloca
+#  else
+#   include <stddef.h>
+#   ifdef  __cplusplus
+extern "C"
+#   endif
+void *alloca (size_t);
+#  endif
+# endif
+#endif
+
+#include "core.h"
+#include "randombytes.h"
+#include "utils.h"
+
 #ifndef ENOSYS
 # define ENOSYS ENXIO
 #endif
 
-#if defined(_WIN32) && (!defined(WINAPI_FAMILY) || WINAPI_FAMILY == WINAPI_FAMILY_DESKTOP_APP)
+#if defined(_WIN32) && \
+    (!defined(WINAPI_FAMILY) || WINAPI_FAMILY == WINAPI_FAMILY_DESKTOP_APP)
 # define WINAPI_DESKTOP
 #endif
 
 #define CANARY_SIZE 16U
-#define GARBAGE_VALUE 0xd0
+#define GARBAGE_VALUE 0xdb
 
 #ifndef MAP_NOCORE
 # define MAP_NOCORE 0
@@ -40,45 +64,63 @@
 #if !defined(MAP_ANON) && defined(MAP_ANONYMOUS)
 # define MAP_ANON MAP_ANONYMOUS
 #endif
-#if defined(WINAPI_DESKTOP) || (defined(MAP_ANON) && defined(HAVE_MMAP)) || defined(HAVE_POSIX_MEMALIGN)
+#if defined(WINAPI_DESKTOP) || (defined(MAP_ANON) && defined(HAVE_MMAP)) || \
+    defined(HAVE_POSIX_MEMALIGN)
 # define HAVE_ALIGNED_MALLOC
 #endif
-#if defined(HAVE_MPROTECT) && !(defined(PROT_NONE) && defined(PROT_READ) && defined(PROT_WRITE))
+#if defined(HAVE_MPROTECT) && \
+    !(defined(PROT_NONE) && defined(PROT_READ) && defined(PROT_WRITE))
 # undef HAVE_MPROTECT
 #endif
-#if defined(HAVE_ALIGNED_MALLOC) && (defined(WINAPI_DESKTOP) || defined(HAVE_MPROTECT))
+#if defined(HAVE_ALIGNED_MALLOC) && \
+    (defined(WINAPI_DESKTOP) || defined(HAVE_MPROTECT))
 # define HAVE_PAGE_PROTECTION
 #endif
-
-static size_t page_size;
-static unsigned char canary[CANARY_SIZE];
-
-#ifdef HAVE_WEAK_SYMBOLS
-__attribute__ ((weak)) void
-_sodium_dummy_symbol_to_prevent_memzero_lto(void * const pnt, const size_t len)
-{
-    (void) pnt;
-    (void) len;
-}
+#if !defined(MADV_DODUMP) && defined(MADV_CORE)
+# define MADV_DODUMP   MADV_CORE
+# define MADV_DONTDUMP MADV_NOCORE
 #endif
 
+static size_t        page_size;
+static unsigned char canary[CANARY_SIZE];
+
+/* LCOV_EXCL_START */
+#ifdef HAVE_WEAK_SYMBOLS
+__attribute__((weak)) void
+_sodium_dummy_symbol_to_prevent_memzero_lto(void *const  pnt,
+                                            const size_t len);
+__attribute__((weak)) void
+_sodium_dummy_symbol_to_prevent_memzero_lto(void *const  pnt,
+                                            const size_t len)
+{
+    (void) pnt; /* LCOV_EXCL_LINE */
+    (void) len; /* LCOV_EXCL_LINE */
+}
+#endif
+/* LCOV_EXCL_STOP */
+
 void
-sodium_memzero(void * const pnt, const size_t len)
+sodium_memzero(void *const pnt, const size_t len)
 {
 #ifdef _WIN32
     SecureZeroMemory(pnt, len);
 #elif defined(HAVE_MEMSET_S)
-    if (memset_s(pnt, (rsize_t) len, 0, (rsize_t) len) != 0) {
-        abort(); /* LCOV_EXCL_LINE */
+    if (len > 0U && memset_s(pnt, (rsize_t) len, 0, (rsize_t) len) != 0) {
+        sodium_misuse(); /* LCOV_EXCL_LINE */
     }
 #elif defined(HAVE_EXPLICIT_BZERO)
     explicit_bzero(pnt, len);
+#elif defined(HAVE_EXPLICIT_MEMSET)
+    explicit_memset(pnt, 0, len);
 #elif HAVE_WEAK_SYMBOLS
     memset(pnt, 0, len);
     _sodium_dummy_symbol_to_prevent_memzero_lto(pnt, len);
+# ifdef HAVE_INLINE_ASM
+    __asm__ __volatile__ ("" : : "r"(pnt) : "memory");
+# endif
 #else
     volatile unsigned char *volatile pnt_ =
-        (volatile unsigned char * volatile) pnt;
+        (volatile unsigned char *volatile) pnt;
     size_t i = (size_t) 0U;
 
     while (i < len) {
@@ -87,11 +129,26 @@ sodium_memzero(void * const pnt, const size_t len)
 #endif
 }
 
+void
+sodium_stackzero(const size_t len)
+{
+#ifdef HAVE_C_VARARRAYS
+    unsigned char fodder[len];
+    sodium_memzero(fodder, len);
+#elif HAVE_ALLOCA
+    sodium_memzero(alloca(len), len);
+#endif
+}
+
 #ifdef HAVE_WEAK_SYMBOLS
-__attribute__ ((weak)) void
+__attribute__((weak)) void
 _sodium_dummy_symbol_to_prevent_memcmp_lto(const unsigned char *b1,
                                            const unsigned char *b2,
-                                           const size_t len)
+                                           const size_t         len);
+__attribute__((weak)) void
+_sodium_dummy_symbol_to_prevent_memcmp_lto(const unsigned char *b1,
+                                           const unsigned char *b2,
+                                           const size_t         len)
 {
     (void) b1;
     (void) b2;
@@ -100,19 +157,19 @@ _sodium_dummy_symbol_to_prevent_memcmp_lto(const unsigned char *b1,
 #endif
 
 int
-sodium_memcmp(const void * const b1_, const void * const b2_, size_t len)
+sodium_memcmp(const void *const b1_, const void *const b2_, size_t len)
 {
 #ifdef HAVE_WEAK_SYMBOLS
     const unsigned char *b1 = (const unsigned char *) b1_;
     const unsigned char *b2 = (const unsigned char *) b2_;
 #else
     const volatile unsigned char *volatile b1 =
-        (const volatile unsigned char * volatile) b1_;
+        (const volatile unsigned char *volatile) b1_;
     const volatile unsigned char *volatile b2 =
-        (const volatile unsigned char * volatile) b2_;
+        (const volatile unsigned char *volatile) b2_;
 #endif
-    size_t               i;
-    unsigned char        d = (unsigned char) 0U;
+    size_t                 i;
+    volatile unsigned char d = 0U;
 
 #if HAVE_WEAK_SYMBOLS
     _sodium_dummy_symbol_to_prevent_memcmp_lto(b1, b2, len);
@@ -124,10 +181,14 @@ sodium_memcmp(const void * const b1_, const void * const b2_, size_t len)
 }
 
 #ifdef HAVE_WEAK_SYMBOLS
-__attribute__ ((weak)) void
+__attribute__((weak)) void
 _sodium_dummy_symbol_to_prevent_compare_lto(const unsigned char *b1,
                                             const unsigned char *b2,
-                                            const size_t len)
+                                            const size_t         len);
+__attribute__((weak)) void
+_sodium_dummy_symbol_to_prevent_compare_lto(const unsigned char *b1,
+                                            const unsigned char *b2,
+                                            const size_t         len)
 {
     (void) b1;
     (void) b2;
@@ -142,14 +203,15 @@ sodium_compare(const unsigned char *b1_, const unsigned char *b2_, size_t len)
     const unsigned char *b1 = b1_;
     const unsigned char *b2 = b2_;
 #else
-    const volatile unsigned char * volatile b1 =
-        (const volatile unsigned char * volatile) b1_;
-    const volatile unsigned char * volatile b2 =
-        (const volatile unsigned char * volatile) b2_;
+    const volatile unsigned char *volatile b1 =
+        (const volatile unsigned char *volatile) b1_;
+    const volatile unsigned char *volatile b2 =
+        (const volatile unsigned char *volatile) b2_;
 #endif
-    unsigned char gt = 0U;
-    unsigned char eq = 1U;
-    size_t        i;
+    size_t                 i;
+    volatile unsigned char gt = 0U;
+    volatile unsigned char eq = 1U;
+    uint16_t               x1, x2;
 
 #if HAVE_WEAK_SYMBOLS
     _sodium_dummy_symbol_to_prevent_compare_lto(b1, b2, len);
@@ -157,8 +219,10 @@ sodium_compare(const unsigned char *b1_, const unsigned char *b2_, size_t len)
     i = len;
     while (i != 0U) {
         i--;
-        gt |= ((b2[i] - b1[i]) >> 8) & eq;
-        eq &= ((b2[i] ^ b1[i]) - 1) >> 8;
+        x1 = b1[i];
+        x2 = b2[i];
+        gt |= ((x2 - x1) >> 8) & eq;
+        eq &= ((x2 ^ x1) - 1) >> 8;
     }
     return (int) (gt + gt + eq) - 1;
 }
@@ -166,8 +230,8 @@ sodium_compare(const unsigned char *b1_, const unsigned char *b2_, size_t len)
 int
 sodium_is_zero(const unsigned char *n, const size_t nlen)
 {
-    size_t        i;
-    unsigned char d = 0U;
+    size_t                 i;
+    volatile unsigned char d = 0U;
 
     for (i = 0U; i < nlen; i++) {
         d |= n[i];
@@ -182,28 +246,30 @@ sodium_increment(unsigned char *n, const size_t nlen)
     uint_fast16_t c = 1U;
 
 #ifdef HAVE_AMD64_ASM
-    uint64_t      t64, t64_2;
-    uint32_t      t32;
+    uint64_t t64, t64_2;
+    uint32_t t32;
 
     if (nlen == 12U) {
-        __asm__ __volatile__("xorq %[t64], %[t64] \n"
-                             "xorl %[t32], %[t32] \n"
-                             "stc \n"
-                             "adcq %[t64], (%[out]) \n"
-                             "adcl %[t32], 8(%[out]) \n"
-                             : [t64] "=&r"(t64), [t32] "=&r" (t32)
-                             : [out] "D"(n)
-                             : "memory", "flags", "cc");
+        __asm__ __volatile__(
+            "xorq %[t64], %[t64] \n"
+            "xorl %[t32], %[t32] \n"
+            "stc \n"
+            "adcq %[t64], (%[out]) \n"
+            "adcl %[t32], 8(%[out]) \n"
+            : [t64] "=&r"(t64), [t32] "=&r"(t32)
+            : [out] "D"(n)
+            : "memory", "flags", "cc");
         return;
     } else if (nlen == 24U) {
-        __asm__ __volatile__("movq $1, %[t64] \n"
-                             "xorq %[t64_2], %[t64_2] \n"
-                             "addq %[t64], (%[out]) \n"
-                             "adcq %[t64_2], 8(%[out]) \n"
-                             "adcq %[t64_2], 16(%[out]) \n"
-                             : [t64] "=&r"(t64), [t64_2] "=&r" (t64_2)
-                             : [out] "D"(n)
-                             : "memory", "flags", "cc");
+        __asm__ __volatile__(
+            "movq $1, %[t64] \n"
+            "xorq %[t64_2], %[t64_2] \n"
+            "addq %[t64], (%[out]) \n"
+            "adcq %[t64_2], 8(%[out]) \n"
+            "adcq %[t64_2], 16(%[out]) \n"
+            : [t64] "=&r"(t64), [t64_2] "=&r"(t64_2)
+            : [out] "D"(n)
+            : "memory", "flags", "cc");
         return;
     } else if (nlen == 8U) {
         __asm__ __volatile__("incq (%[out]) \n"
@@ -223,130 +289,93 @@ sodium_increment(unsigned char *n, const size_t nlen)
 void
 sodium_add(unsigned char *a, const unsigned char *b, const size_t len)
 {
-    size_t        i = 0U;
+    size_t        i;
     uint_fast16_t c = 0U;
 
 #ifdef HAVE_AMD64_ASM
-    uint64_t      t64, t64_2, t64_3;
-    uint32_t      t32;
+    uint64_t t64, t64_2, t64_3;
+    uint32_t t32;
 
     if (len == 12U) {
-        __asm__ __volatile__("movq (%[in]), %[t64] \n"
-                             "movl 8(%[in]), %[t32] \n"
-                             "addq %[t64], (%[out]) \n"
-                             "adcl %[t32], 8(%[out]) \n"
-                             : [t64] "=&r"(t64), [t32] "=&r" (t32)
-                             : [in] "S"(b), [out] "D"(a)
-                             : "memory", "flags", "cc");
+        __asm__ __volatile__(
+            "movq (%[in]), %[t64] \n"
+            "movl 8(%[in]), %[t32] \n"
+            "addq %[t64], (%[out]) \n"
+            "adcl %[t32], 8(%[out]) \n"
+            : [t64] "=&r"(t64), [t32] "=&r"(t32)
+            : [in] "S"(b), [out] "D"(a)
+            : "memory", "flags", "cc");
         return;
     } else if (len == 24U) {
-        __asm__ __volatile__("movq (%[in]), %[t64] \n"
-                             "movq 8(%[in]), %[t64_2] \n"
-                             "movq 16(%[in]), %[t64_3] \n"
-                             "addq %[t64], (%[out]) \n"
-                             "adcq %[t64_2], 8(%[out]) \n"
-                             "adcq %[t64_3], 16(%[out]) \n"
-                             : [t64] "=&r"(t64), [t64_2] "=&r"(t64_2), [t64_3] "=&r"(t64_3)
-                             : [in] "S"(b), [out] "D"(a)
-                             : "memory", "flags", "cc");
+        __asm__ __volatile__(
+            "movq (%[in]), %[t64] \n"
+            "movq 8(%[in]), %[t64_2] \n"
+            "movq 16(%[in]), %[t64_3] \n"
+            "addq %[t64], (%[out]) \n"
+            "adcq %[t64_2], 8(%[out]) \n"
+            "adcq %[t64_3], 16(%[out]) \n"
+            : [t64] "=&r"(t64), [t64_2] "=&r"(t64_2), [t64_3] "=&r"(t64_3)
+            : [in] "S"(b), [out] "D"(a)
+            : "memory", "flags", "cc");
         return;
     } else if (len == 8U) {
-        __asm__ __volatile__("movq (%[in]), %[t64] \n"
-                             "addq %[t64], (%[out]) \n"
-                             : [t64] "=&r"(t64)
-                             : [in] "S"(b), [out] "D"(a)
-                             : "memory", "flags", "cc");
+        __asm__ __volatile__(
+            "movq (%[in]), %[t64] \n"
+            "addq %[t64], (%[out]) \n"
+            : [t64] "=&r"(t64)
+            : [in] "S"(b), [out] "D"(a)
+            : "memory", "flags", "cc");
         return;
     }
 #endif
-    for (; i < len; i++) {
+    for (i = 0U; i < len; i++) {
         c += (uint_fast16_t) a[i] + (uint_fast16_t) b[i];
         a[i] = (unsigned char) c;
         c >>= 8;
     }
 }
 
-/* Derived from original code by CodesInChaos */
-char *
-sodium_bin2hex(char * const hex, const size_t hex_maxlen,
-               const unsigned char * const bin, const size_t bin_len)
+void
+sodium_sub(unsigned char *a, const unsigned char *b, const size_t len)
 {
-    size_t       i = (size_t) 0U;
-    unsigned int x;
-    int          b;
-    int          c;
+    uint_fast16_t c = 0U;
+    size_t        i;
 
-    if (bin_len >= SIZE_MAX / 2 || hex_maxlen <= bin_len * 2U) {
-        abort(); /* LCOV_EXCL_LINE */
-    }
-    while (i < bin_len) {
-        c = bin[i] & 0xf;
-        b = bin[i] >> 4;
-        x = (unsigned char) (87U + c + (((c - 10U) >> 8) & ~38U)) << 8 |
-            (unsigned char) (87U + b + (((b - 10U) >> 8) & ~38U));
-        hex[i * 2U] = (char) x;
-        x >>= 8;
-        hex[i * 2U + 1U] = (char) x;
-        i++;
-    }
-    hex[i * 2U] = 0U;
+#ifdef HAVE_AMD64_ASM
+    uint64_t t64_1, t64_2, t64_3, t64_4;
+    uint64_t t64_5, t64_6, t64_7, t64_8;
+    uint32_t t32;
 
-    return hex;
-}
-
-int
-sodium_hex2bin(unsigned char * const bin, const size_t bin_maxlen,
-               const char * const hex, const size_t hex_len,
-               const char * const ignore, size_t * const bin_len,
-               const char ** const hex_end)
-{
-    size_t        bin_pos = (size_t) 0U;
-    size_t        hex_pos = (size_t) 0U;
-    int           ret = 0;
-    unsigned char c;
-    unsigned char c_acc = 0U;
-    unsigned char c_alpha0, c_alpha;
-    unsigned char c_num0, c_num;
-    unsigned char c_val;
-    unsigned char state = 0U;
-
-    while (hex_pos < hex_len) {
-        c = (unsigned char) hex[hex_pos];
-        c_num = c ^ 48U;
-        c_num0 = (c_num - 10U) >> 8;
-        c_alpha = (c & ~32U) - 55U;
-        c_alpha0 = ((c_alpha - 10U) ^ (c_alpha - 16U)) >> 8;
-        if ((c_num0 | c_alpha0) == 0U) {
-            if (ignore != NULL && state == 0U && strchr(ignore, c) != NULL) {
-                hex_pos++;
-                continue;
-            }
-            break;
-        }
-        c_val = (c_num0 & c_num) | (c_alpha0 & c_alpha);
-        if (bin_pos >= bin_maxlen) {
-            ret = -1;
-            errno = ERANGE;
-            break;
-        }
-        if (state == 0U) {
-            c_acc = c_val * 16U;
-        } else {
-            bin[bin_pos++] = c_acc | c_val;
-        }
-        state = ~state;
-        hex_pos++;
+    if (len == 64U) {
+        __asm__ __volatile__(
+            "movq   (%[in]), %[t64_1] \n"
+            "movq  8(%[in]), %[t64_2] \n"
+            "movq 16(%[in]), %[t64_3] \n"
+            "movq 24(%[in]), %[t64_4] \n"
+            "movq 32(%[in]), %[t64_5] \n"
+            "movq 40(%[in]), %[t64_6] \n"
+            "movq 48(%[in]), %[t64_7] \n"
+            "movq 56(%[in]), %[t64_8] \n"
+            "subq %[t64_1],   (%[out]) \n"
+            "sbbq %[t64_2],  8(%[out]) \n"
+            "sbbq %[t64_3], 16(%[out]) \n"
+            "sbbq %[t64_4], 24(%[out]) \n"
+            "sbbq %[t64_5], 32(%[out]) \n"
+            "sbbq %[t64_6], 40(%[out]) \n"
+            "sbbq %[t64_7], 48(%[out]) \n"
+            "sbbq %[t64_8], 56(%[out]) \n"
+            : [t64_1] "=&r"(t64_1), [t64_2] "=&r"(t64_2), [t64_3] "=&r"(t64_3), [t64_4] "=&r"(t64_4),
+              [t64_5] "=&r"(t64_5), [t64_6] "=&r"(t64_6), [t64_7] "=&r"(t64_7), [t64_8] "=&r"(t64_8)
+            : [in] "S"(b), [out] "D"(a)
+            : "memory", "flags", "cc");
+        return;
     }
-    if (state != 0U) {
-        hex_pos--;
+#endif
+    for (i = 0U; i < len; i++) {
+        c = (uint_fast16_t) a[i] - (uint_fast16_t) b[i] - c;
+        a[i] = (unsigned char) c;
+        c = (c >> 8) & 1U;
     }
-    if (hex_end != NULL) {
-        *hex_end = &hex[hex_pos];
-    }
-    if (bin_len != NULL) {
-        *bin_len = bin_pos;
-    }
-    return ret;
 }
 
 int
@@ -364,7 +393,7 @@ _sodium_alloc_init(void)
     page_size = (size_t) si.dwPageSize;
 # endif
     if (page_size < CANARY_SIZE || page_size < sizeof(size_t)) {
-        abort(); /* LCOV_EXCL_LINE */
+        sodium_misuse(); /* LCOV_EXCL_LINE */
     }
 #endif
     randombytes_buf(canary, sizeof canary);
@@ -373,7 +402,7 @@ _sodium_alloc_init(void)
 }
 
 int
-sodium_mlock(void * const addr, const size_t len)
+sodium_mlock(void *const addr, const size_t len)
 {
 #if defined(MADV_DONTDUMP) && defined(HAVE_MADVISE)
     (void) madvise(addr, len, MADV_DONTDUMP);
@@ -389,7 +418,7 @@ sodium_mlock(void * const addr, const size_t len)
 }
 
 int
-sodium_munlock(void * const addr, const size_t len)
+sodium_munlock(void *const addr, const size_t len)
 {
     sodium_memzero(addr, len);
 #if defined(MADV_DODUMP) && defined(HAVE_MADVISE)
@@ -449,7 +478,7 @@ _mprotect_readwrite(void *ptr, size_t size)
 
 #ifdef HAVE_ALIGNED_MALLOC
 
-__attribute__ ((noreturn)) static void
+__attribute__((noreturn)) static void
 _out_of_bounds(void)
 {
 # ifdef SIGSEGV
@@ -457,7 +486,7 @@ _out_of_bounds(void)
 # elif defined(SIGKILL)
     raise(SIGKILL);
 # endif
-    abort();
+    abort(); /* not something we want any higher-level API to catch */
 } /* LCOV_EXCL_LINE */
 
 static inline size_t
@@ -468,20 +497,21 @@ _page_round(const size_t size)
     return (size + page_mask) & ~page_mask;
 }
 
-static __attribute__ ((malloc)) unsigned char *
+static __attribute__((malloc)) unsigned char *
 _alloc_aligned(const size_t size)
 {
     void *ptr;
 
 # if defined(MAP_ANON) && defined(HAVE_MMAP)
     if ((ptr = mmap(NULL, size, PROT_READ | PROT_WRITE,
-                    MAP_ANON | MAP_PRIVATE | MAP_NOCORE, -1, 0)) == MAP_FAILED) {
+                    MAP_ANON | MAP_PRIVATE | MAP_NOCORE, -1, 0)) ==
+        MAP_FAILED) {
         ptr = NULL; /* LCOV_EXCL_LINE */
-    } /* LCOV_EXCL_LINE */
+    }               /* LCOV_EXCL_LINE */
 # elif defined(HAVE_POSIX_MEMALIGN)
     if (posix_memalign(&ptr, page_size, size) != 0) {
         ptr = NULL; /* LCOV_EXCL_LINE */
-    } /* LCOV_EXCL_LINE */
+    }               /* LCOV_EXCL_LINE */
 # elif defined(WINAPI_DESKTOP)
     ptr = VirtualAlloc(NULL, size, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
 # else
@@ -491,7 +521,7 @@ _alloc_aligned(const size_t size)
 }
 
 static void
-_free_aligned(unsigned char * const ptr, const size_t size)
+_free_aligned(unsigned char *const ptr, const size_t size)
 {
 # if defined(MAP_ANON) && defined(HAVE_MMAP)
     (void) munmap(ptr, size);
@@ -501,11 +531,11 @@ _free_aligned(unsigned char * const ptr, const size_t size)
     VirtualFree(ptr, 0U, MEM_RELEASE);
 # else
 #  error Bug
-# endif
+#endif
 }
 
 static unsigned char *
-_unprotected_ptr_from_user_ptr(void * const ptr)
+_unprotected_ptr_from_user_ptr(void *const ptr)
 {
     uintptr_t      unprotected_ptr_u;
     unsigned char *canary_ptr;
@@ -515,7 +545,7 @@ _unprotected_ptr_from_user_ptr(void * const ptr)
     page_mask = page_size - 1U;
     unprotected_ptr_u = ((uintptr_t) canary_ptr & (uintptr_t) ~page_mask);
     if (unprotected_ptr_u <= page_size * 2U) {
-        abort(); /* LCOV_EXCL_LINE */
+        sodium_misuse(); /* LCOV_EXCL_LINE */
     }
     return (unsigned char *) unprotected_ptr_u;
 }
@@ -523,13 +553,13 @@ _unprotected_ptr_from_user_ptr(void * const ptr)
 #endif /* HAVE_ALIGNED_MALLOC */
 
 #ifndef HAVE_ALIGNED_MALLOC
-static __attribute__ ((malloc)) void *
+static __attribute__((malloc)) void *
 _sodium_malloc(const size_t size)
 {
     return malloc(size > (size_t) 0U ? size : (size_t) 1U);
 }
 #else
-static __attribute__ ((malloc)) void *
+static __attribute__((malloc)) void *
 _sodium_malloc(const size_t size)
 {
     void          *user_ptr;
@@ -545,11 +575,11 @@ _sodium_malloc(const size_t size)
         return NULL;
     }
     if (page_size <= sizeof canary || page_size < sizeof unprotected_size) {
-        abort(); /* LCOV_EXCL_LINE */
+        sodium_misuse(); /* LCOV_EXCL_LINE */
     }
     size_with_canary = (sizeof canary) + size;
     unprotected_size = _page_round(size_with_canary);
-    total_size = page_size + page_size + unprotected_size + page_size;
+    total_size       = page_size + page_size + unprotected_size + page_size;
     if ((base_ptr = _alloc_aligned(total_size)) == NULL) {
         return NULL; /* LCOV_EXCL_LINE */
     }
@@ -560,8 +590,8 @@ _sodium_malloc(const size_t size)
 # endif
     _mprotect_noaccess(unprotected_ptr + unprotected_size, page_size);
     sodium_mlock(unprotected_ptr, unprotected_size);
-    canary_ptr = unprotected_ptr + _page_round(size_with_canary) -
-        size_with_canary;
+    canary_ptr =
+        unprotected_ptr + _page_round(size_with_canary) - size_with_canary;
     user_ptr = canary_ptr + sizeof canary;
     memcpy(canary_ptr, canary, sizeof canary);
     memcpy(base_ptr, &unprotected_size, sizeof unprotected_size);
@@ -572,7 +602,7 @@ _sodium_malloc(const size_t size)
 }
 #endif /* !HAVE_ALIGNED_MALLOC */
 
-__attribute__ ((malloc)) void *
+__attribute__((malloc)) void *
 sodium_malloc(const size_t size)
 {
     void *ptr;
@@ -585,18 +615,14 @@ sodium_malloc(const size_t size)
     return ptr;
 }
 
-__attribute__ ((malloc)) void *
+__attribute__((malloc)) void *
 sodium_allocarray(size_t count, size_t size)
 {
-    size_t total_size;
-
     if (count > (size_t) 0U && size >= (size_t) SIZE_MAX / count) {
         errno = ENOMEM;
         return NULL;
     }
-    total_size = count * size;
-
-    return sodium_malloc(total_size);
+    return sodium_malloc(count * size);
 }
 
 #ifndef HAVE_ALIGNED_MALLOC
@@ -618,9 +644,9 @@ sodium_free(void *ptr)
     if (ptr == NULL) {
         return;
     }
-    canary_ptr = ((unsigned char *) ptr) - sizeof canary;
+    canary_ptr      = ((unsigned char *) ptr) - sizeof canary;
     unprotected_ptr = _unprotected_ptr_from_user_ptr(ptr);
-    base_ptr = unprotected_ptr - page_size * 2U;
+    base_ptr        = unprotected_ptr - page_size * 2U;
     memcpy(&unprotected_size, base_ptr, sizeof unprotected_size);
     total_size = page_size + page_size + unprotected_size + page_size;
     _mprotect_readwrite(base_ptr, total_size);
@@ -628,8 +654,8 @@ sodium_free(void *ptr)
         _out_of_bounds();
     }
 # ifndef HAVE_PAGE_PROTECTION
-    if (sodium_memcmp(unprotected_ptr + unprotected_size,
-                      canary, sizeof canary) != 0) {
+    if (sodium_memcmp(unprotected_ptr + unprotected_size, canary,
+                      sizeof canary) != 0) {
         _out_of_bounds();
     }
 # endif
@@ -656,7 +682,7 @@ _sodium_mprotect(void *ptr, int (*cb)(void *ptr, size_t size))
     size_t         unprotected_size;
 
     unprotected_ptr = _unprotected_ptr_from_user_ptr(ptr);
-    base_ptr = unprotected_ptr - page_size * 2U;
+    base_ptr        = unprotected_ptr - page_size * 2U;
     memcpy(&unprotected_size, base_ptr, sizeof unprotected_size);
 
     return cb(unprotected_ptr, unprotected_size);
@@ -680,3 +706,74 @@ sodium_mprotect_readwrite(void *ptr)
 {
     return _sodium_mprotect(ptr, _mprotect_readwrite);
 }
+
+int
+sodium_pad(size_t *padded_buflen_p, unsigned char *buf,
+           size_t unpadded_buflen, size_t blocksize, size_t max_buflen)
+{
+    unsigned char          *tail;
+    size_t                  i;
+    size_t                  xpadlen;
+    size_t                  xpadded_len;
+    volatile unsigned char  mask;
+    unsigned char           barrier_mask;
+
+    if (blocksize <= 0U) {
+        return -1;
+    }
+    xpadlen = blocksize - 1U;
+    if ((blocksize & (blocksize - 1U)) == 0U) {
+        xpadlen -= unpadded_buflen & (blocksize - 1U);
+    } else {
+        xpadlen -= unpadded_buflen % blocksize;
+    }
+    if ((size_t) SIZE_MAX - unpadded_buflen <= xpadlen) {
+        sodium_misuse();
+    }
+    xpadded_len = unpadded_buflen + xpadlen;
+    if (xpadded_len >= max_buflen) {
+        return -1;
+    }
+    tail = &buf[xpadded_len];
+    if (padded_buflen_p != NULL) {
+        *padded_buflen_p = xpadded_len + 1U;
+    }
+    mask = 0U;
+    for (i = 0; i < blocksize; i++) {
+        barrier_mask = (unsigned char) (((i ^ xpadlen) - 1U)
+           >> ((sizeof(size_t) - 1) * CHAR_BIT));
+        *(tail - i) = ((*(tail - i)) & mask) | (0x80 & barrier_mask);
+        mask |= barrier_mask;
+    }
+    return 0;
+}
+
+int
+sodium_unpad(size_t *unpadded_buflen_p, const unsigned char *buf,
+             size_t padded_buflen, size_t blocksize)
+{
+    const unsigned char *tail;
+    unsigned char        acc = 0U;
+    unsigned char        c;
+    unsigned char        valid = 0U;
+    volatile size_t      pad_len = 0U;
+    size_t               i;
+    size_t               is_barrier;
+
+    if (padded_buflen < blocksize || blocksize <= 0U) {
+        return -1;
+    }
+    tail = &buf[padded_buflen - 1U];
+
+    for (i = 0U; i < blocksize; i++) {
+        c = *(tail - i);
+        is_barrier =
+            (( (acc - 1U) & (pad_len - 1U) & ((c ^ 0x80) - 1U) ) >> 8) & 1U;
+        acc |= c;
+        pad_len |= i & (1U + ~is_barrier);
+        valid |= (unsigned char) is_barrier;
+    }
+    *unpadded_buflen_p = padded_buflen - 1U - pad_len;
+
+    return (int) (valid - 1U);
+}
